;--------------------------------------------------------------------------
; HYPER VIPER
;
; Written by Kenton Price in 1986
; Enhancements for release by Paul Davis
; Joystick support by Pitfall Jones
; Atom version by Kees van Oss
; Dragon/CoCo port by Phill Harvey-Smith 2011-08.
;
; Version 1.1
; Released by Retro Software, January 2011
; http://www.retrosoftware.co.uk/hyperviper
;--------------------------------------------------------------------------
; This work is licensed by Kenton Price (retrosoftware@kentonprice.com)
; under the Creative Commons Attribution-NonCommercial-ShareAlike
; 3.0 Unported License.
;
; To view a copy of this licence, visit
; http://creativecommons.org/licenses/by-nc-sa/3.0/
; or send a letter to Creative Commons, 171 Second Street,
; Suite 300, San Francisco, California 94105, USA.
;--------------------------------------------------------------------------

start_prog:
	lbra		main

; sprite/score data
rock_sprite_data             equ $70
mushroom_sprite_data         equ $90
brick_wall_sprite_data       equ $A0
snake_body_sprite_data       equ $B0
snake_head_sprite_data       equ $c0
snake_head_right_sprite_data equ $D0

; game constants
max_lives_count             equ 13
max_screen_number           equ 5
initial_snake_segment_count equ 5
initial_lives_count         equ 5
initial_flash_duration      equ 30
initial_sound_flag          equ 0
game_speed                  equ 5
mushroom_count              equ 20
safe_zone_size              equ 3
wait_time                   equ 50

; game variables
snake_head_ptr              fdb	0
sprite_data_ptr				fdb	0	; pointer to data to put on screen
sprite_screen_ptr			fdb	0	; pointer to screen to draw sprite
safe_zone_left_ptr          fdb	0
safe_zone_right_ptr         fdb	0
high_score_entry_ptr        fdb	0
high_score_copy_ptr         fdb	0
tune_ptr                    fdb	0
sprite_base_ptr				fdb	0	; Pointer to the base of sprite set in use

snake_segment_pointers 		zmd	$20	; space for 32 segments
score                       fcb	0,0,0,0	; 4 digits

snake_segment_count         fcb 0
snake_direction             fcb 0
pause_flag                  fcb 0,0
collision_object            fcb 0,0,0,0,0
last_screen_number          fcb 0
timer0                      fcb 0
timer1                      fcb 0
timer2                      fcb 0
sound_flag                  fcb 0,0,0,0,0
flash_counter               fcb 0,0
flash_duration              fcb 0,0
high_score_crsr_pos         fcb 0
high_score_entry_pos        fcb 0
last_rock_ptr               fcb 0,0
counter                     fcb 0
current_screen_number       fcb 0
lives_counter               fcb 0
wait_counter				fcb	0
last_key					fcb	0

temp_b    					fcb 0,0
w_temp_store0   			fdb 0
w_temp_store2   			fdb 0

mono_or_colour				fcb	0

;Colour constants
green    equ $00
yellow   equ $40
blue     equ $80
red      equ $C0

mono_mode					equ	0
colour_mode					equ	1

; screen constants
bytes_per_character         equ 8
characters_per_line         equ 32
bytes_per_line              equ bytes_per_character*characters_per_line

lives_screen_location       fdb 0
flashing_mushroom_location  fdb 0	
snake_start_location        fdb 0

lives_xpos					equ	24
lives_ypos					equ	0
lives_screen_offset			equ lives_xpos+(lives_ypos*bytes_per_line)
snake_start_xpos			equ	12
snake_start_ypos			equ	14
snake_start_offset			equ snake_start_xpos+(snake_start_ypos*bytes_per_line)

play_area_top_line			equ	3
play_area_offset			equ play_area_top_line*bytes_per_line
top_wall_offset				equ play_area_offset
bottom_wall_offset			equ	23*bytes_per_line
left_wall_offset			equ 0+play_area_offset
right_wall_offset			equ 31+play_area_offset

; sprite image pointers
sprite_img_ofs				equ	2
mushroom_img_ofs			equ	mushroom_sprite_data+sprite_img_ofs
rock_img_ofs				equ	rock_sprite_data+sprite_img_ofs
wall_img_ofs				equ	brick_wall_sprite_data+sprite_img_ofs

mushroom_image_ptr          fdb	0
rock_image_ptr              fdb	0
wall_image_ptr              fdb	0

; movement direction constants
dir_left                    equ 1
dir_right                   equ 2
dir_up                      equ 3
dir_down                    equ 4

;keys
key_up						equ	$51	; Q-key
key_down					equ $41 ; A-key
key_left					equ $4f ; O-key
key_right					equ $50 ; P-key
key_pause					equ	$0c	; clear key
key_start					equ	$20	; space key
key_test					equ	$54 ; T key

;-------------------------------------------------------------
;MAIN
;-------------------------------------------------------------

main:
dwnld:
	; entry point to the game
	; initialise screen display

	LDA 	#initial_sound_flag
	STA 	sound_flag
	LDA 	#1
	STA 	last_screen_number

; Dragon / CoCo initialisation
	lbsr	detect_machine_type		; detect the machine type
	lbsr	select_colourset		; Prompt and select colourset
	
	tst		mono_or_colour			; Which colourset
	bne		select_colour

	leax	sprites_mono,pcr		; setup sprite ptr
	stx		sprite_base_ptr

	ldx		#sprites_mono+mushroom_img_ofs
	stx		mushroom_image_ptr
	ldx		#sprites_mono+rock_img_ofs
	stx		rock_image_ptr
	ldx		#sprites_mono+wall_img_ofs
	stx		wall_image_ptr
	

	lda		#4						; Mono : pmode 4
	bra		set_colourset			
	
select_colour:
	leax	sprites_colour,pcr		; setup sprite ptr
	stx		sprite_base_ptr
	
	ldx		#sprites_colour+mushroom_img_ofs
	stx		mushroom_image_ptr
	ldx		#sprites_colour+rock_img_ofs
	stx		rock_image_ptr
	ldx		#sprites_colour+wall_img_ofs
	stx		wall_image_ptr

	lda		#3						; Colour pmode 3
set_colourset:
	sta		>GrCurrPmode
	
	lda		#6						; graphics page 1
	sta		>GrStartPages
	
	tst		>dragon_or_coco			; check machine type
	bne		SetCoCo
	
	andcc	#~FlagZero				; graphics mode
	jsr		DGrSelectDisplay		; set graphics mode
	bra		SetAddrs				
SetCoCo:
	jsr		CGrSelectDisplay		; set graphics mode

SetAddrs:
	ldx		GrDisplayStartAddr		; Graphic screen base pointer
	leax	lives_screen_offset,x	; Work out pointer to lives
	stx		lives_screen_location

	ldx		GrDisplayStartAddr		; Graphic screen base pointer
	leax	snake_start_offset,x	; work out pointer to snake
	stx		>snake_start_location,pcr 


;-------------------------------------------------------------
title_screen:
	; swap between title screen and high score screen
	; until space bar is pressed
	LDA 	#0
show_screen:
	clr		>timer0
;	inc		>timer0
	clr 	>timer1
	LDA 	>screen_duration
	STA 	>timer2
tsloop:
	LDA 	>timer0
	anda 	#1
	BNE 	high
	lbsr 	display_title_screen
	bra		waitkey
high:
	lbsr	display_high_scores
waitkey:
	lbsr	display_press_space
	LDA 	#1
	STA 	current_screen_number
walp:
;joy:
;	LDA #1
;	STA use_joystick
;	BEQ nofire
;	lbsr check_fire
;	BCS play
;	LDA #0
;	STA use_joystick
	; check for space bar
nofire:
	lbsr 	osrdch
	beq		nokey
	cmpa 	#key_start		; space
	BEQ 	play

	cmpa	#key_test
	lbeq	hs_test

nokey:
	lbsr	oswaitsync
	DEC 	>timer1
	BNE 	walp
	DEC 	>timer2
	BNE 	walp
	LDA 	>screen_duration
	STA 	>timer2
    INC 	>timer0
	bra 	tsloop
;-------------------------------------------------------------
play:
	; check which screen the player got up to last time
	; if >1 offer the choice of screen to play
	LDA		 >last_screen_number
	cmpa 	#1
	BEQ 	le1
	LDY 	#0
levv:
	lbsr	display_which_screen
	
	LDA 	>last_screen_number
	adda	#'0
    ldb 	#24
	stb		>XPOS
    ldb 	#18
	stb		>YPOS
    lbsr	PUTCHAR

	LDA 	>last_screen_number
	adda	#'1
	STA 	>last_screen_number
log:
	lbsr 	osrdch
    adda 	#$20
    cmpa 	#'1
	BMI 	log
	cmpa	>last_screen_number
	BPL 	log
	
	suba	#'0
	STA 	>current_screen_number
	LDA 	>last_screen_number
	suba	#'1
	STA 	>last_screen_number
	; display status panel
le1:
	lbsr	display_status_panel
	; reset game variables
	clr 	score,pcr
	clr 	score+1,pcr
	clr 	score+2,pcr
	clr 	score+3,pcr
	LDA 	#initial_flash_duration
	STA 	flash_duration
	LDA 	#initial_lives_count
	STA 	lives_counter
prsn:

;-------------------------------------------------------------
go:
	clr		pause_flag

;	LDA #<tune_data
;	STA tune_ptr
;	LDA #>tune_data
;	STA tune_ptr+1

    ; Draw snake head in playfield  
	ldd 	snake_start_location
	std 	snake_head_ptr
	lbsr	draw_snake_head

	LDA 	#initial_snake_segment_count
	STA 	snake_segment_count
	LDA 	#dir_left
	STA 	snake_direction
    ; Draw snake body segments in playfield

	ldb		#initial_snake_segment_count-1	; do 8 segments
	ldx		>snake_start_location,pcr		; get location of first segment
	leax	1,x
	leau	snake_segment_pointers+2,pcr	; point to segment data
wcr:
	stx		sprite_screen_ptr
	lbsr	draw_snake_segment
	stx		,u++							; update segment
	leax	1,x								; increment x co-ordinate
	decb									; decrement >counter
	bne		wcr								

    ; Draw brick wall around edge of screen
	ldx		#top_wall_offset
	LDA 	#32						; 32 bricks
	lbsr 	draw_horizontal_wall    ; Top
	
	ldx		#bottom_wall_offset
	LDA 	#32						; 32 bricks
	lbsr	draw_horizontal_wall    ; Bottom
	
	ldx		#left_wall_offset
    LDA 	#20						; 20 bricks
	lbsr	draw_vertical_wall      ; Left
		
	ldx		#right_wall_offset
    LDA 	#20						; 20 bricks
	lbsr	draw_vertical_wall      ; Right


    ; Check screen number to start
	LDA 	current_screen_number
	cmpa 	#1
	BNE 	kk
	lbra	mushy
kk:
	cmpa	#2
	BEQ 	scr2
	cmpa	#3
	BEQ 	scr3
	cmpa	#4
	BEQ 	scr4
	cmpa	#5
	BPL 	scr5

scr2:
	ldx		#(16+3*$100)     		;Draw screen 2
	LDA 	#7
	lbsr	draw_vertical_wall
	
	ldx		#(16+16*$100)
	LDA 	#7
	lbsr	draw_vertical_wall
	lbra	mushy

scr3:
	ldx		#(10+3*$100)     		;Draw screen 3
	LDA 	#7
	lbsr	draw_vertical_wall
	
	ldx		#(10+16*$100)
	LDA 	#7
	lbsr 	draw_vertical_wall
	
	LDX		#(21+3*$100)
	LDA 	#7
	lbsr	draw_vertical_wall
	
	LDX		#(21+16*$100)
	LDA 	#7
	lbsr	draw_vertical_wall
	lbra	mushy

scr4:
	ldx		#(1+13*$100)     		;Draw screen 4
	LDA 	#10
	lbsr 	draw_horizontal_wall
	
	ldx		#(21+13*$100)
	LDA 	#10
	lbsr	draw_horizontal_wall
	
	ldx		#(16+3*$100)
	LDA 	#7
	lbsr 	draw_vertical_wall
	
	ldx		#(16+15*$100)
	LDA 	#8
	lbsr 	draw_vertical_wall
	lbra	mushy
	
scr5:
	ldx		#(7+3*$100)     	;Draw screen 5
	LDA 	#17
	lbsr	draw_vertical_wall
	
	ldx		#(24+7*$100)
	LDA 	#17
	lbsr	draw_vertical_wall
	
	LDX		#(12+7*$100)
	LDA 	#12
	LBSR	draw_horizontal_wall
	
	LDX		#(8+11*$100)
	LDA 	#12
	LBSR	draw_horizontal_wall
	
	LDX		#(12+15*$100)
	LDA 	#12
	LBSR	draw_horizontal_wall
	
	ldx		#(8+19*$100)
	LDA 	#12
	lbsr	draw_horizontal_wall

;-------------------------------------------------------------
mushy:
    ; Draw snake with number of lives left
	ldx 	lives_screen_location
	lda		lives_counter
mushy2:
	stx		sprite_screen_ptr
	lbsr	draw_snake_segment
	leax	-1,x
	deca
	BNE 	mushy2
	
	ldx		#snake_head_right_sprite_data
	stx 	sprite_data_ptr
	ldx		lives_screen_location
	stx 	sprite_screen_ptr
	lbsr 	draw_sprite

    ; Set snake head position
	ldx		snake_start_location
	stx		snake_head_ptr
	stx 	snake_segment_pointers

    ; Place mushrooms at random locations on screen
	ldb		#mushroom_count
mush:
	lbsr	get_random_position
	; Don't place mushroom if something already there
	LDX 	sprite_screen_ptr
	leax	$40,x
	lda		,x
	BNE 	mush
	; Prevent mushrooms from being placed next to each other
	; mushroom in valid position
	lbsr	display_mushroom
	decb		
	BNE 	mush

	LDA 	flash_duration
	STA 	flash_counter

    ; Place rocks at random locations on screen
	LDA 	#4
	adda 	current_screen_number
	adda 	current_screen_number
rloop:
	lbsr 	place_rock
	deca
	BNE 	rloop

    ; Save last rock position
	ldd 	sprite_screen_ptr
	std		last_rock_ptr
	
    ; 1 sec delay
	LDA 	#wait_time
	STA 	wait_counter
wawaw:
	lbsr 	oswaitsync
	DEC 	wait_counter
	BNE 	wawaw

;        lbsr $ffe3

;-------------------------------------------------------------
; Game loop if not dead 
;-------------------------------------------------------------
eachgo:        
;	lda 	$b002
;	eor 	#4
;	sta 	$b002

    ; Turn the last placed rock into a flashing mushroom
	ldd 	last_rock_ptr
	std 	sprite_screen_ptr
	lbsr 	oswaitsync
	lbsr	display_mushroom		; display flashing mushroom
	lbsr	oswaitsync				; wait for a sync
	lbsr	delete_sprite			; delete it again.

    ; Start keyscan
	lbsr	osrdch
	sta		last_key
check_left:
	cmpa 	#key_left
	BNE 	check_right
	
	LDA 	snake_direction 		; ignore attempt to move the snake back on itself
	cmpa 	#dir_right
	BEQ 	check_right
	
	LDA 	#dir_left
	STA 	snake_direction
	BNE 	move

check_right:
	cmpa	#key_right
	BNE 	check_up
	
	LDA 	snake_direction 		; ignore attempt to move the snake back on itself
	cmpa	#dir_left
	BEQ 	check_up
	
	LDA 	#dir_right
	STA 	snake_direction
	BNE 	move
	
check_up:
	cmpa	#key_up
	BNE 	check_down
	
	LDA 	snake_direction 		; ignore attempt to move the snake back on itself
	cmpa	#dir_down
	BEQ 	check_down
	
	LDA 	#dir_up
	STA 	snake_direction
	BNE 	move
	
check_down:
	cmpa	#key_down
	BNE 	move
	LDA 	snake_direction 		; ignore attempt to move the snake back on itself
	cmpa	#dir_up
	BEQ 	move
	LDA 	#dir_down
	STA 	snake_direction

    ; Move snake in correct direction
move:
	lbsr	move_snake_forwards
	LDA 	snake_direction
	cmpa	#dir_left
	BEQ 	left

	cmpa	#dir_right
	BEQ 	right
	
	cmpa	#dir_up
	BEQ 	up
	bra		down
	
left:
	dec 	snake_head_ptr+1
	bra 	goon

right:
	inc 	snake_head_ptr+1
	bra		goon

up:
	dec 	snake_head_ptr
	bra		goon
	
down:
	inc 	snake_head_ptr

goon:
	ldd 	snake_head_ptr
	std 	snake_segment_pointers
	
	; check if snake has collided with anything
	ldx		snake_head_ptr
	lda		$40,x
	STA 	collision_object


	; draw snake head in current direction
	; calculate sprite address snake head left/right/up/down
	LDA 	snake_direction
	ASLA
	ASLA
	ASLA
	ASLA
	ldx		sprite_base_ptr
	leax	snake_body_sprite_data,x
	leax	a,x
	stx		sprite_data_ptr
	lbsr	draw_snake_head_dir

	; draw snake segment over the head in previous location
	ldd 	snake_segment_pointers+2
	std 	sprite_screen_ptr
	lbsr 	draw_snake_segment

	; delete the last snake segment at the tail
	LDA 	snake_segment_count
	ASLA
	leau	snake_segment_pointers,pcr
	leau	a,u
	ldd		,u
	std 	sprite_screen_ptr
	lbsr 	delete_sprite

    ; Check collision status
	LDA 	collision_object
	BEQ 	ok3
	
	cmpa	[mushroom_image_ptr] 		; mushroom - eat
	BEQ 	eat_mushroom
	
	LDA 	collision_object
	cmpa	[rock_image_ptr] 			; rock - check if flashing first
	BNE 	deda 						; anything else - die
	ldd 	last_rock_ptr
	cmpd	flashing_mushroom_location
	BEQ 	eat_mushroom 				; flashing mushroom - eat
deda:
	lbra 	dead
;-------------------------------------------------------------
eat_mushroom:
	lbsr 	eat_sound

	lbsr	place_rock
	; set this new rock to be the potential flashing mushroom
	; unless there is still one on the screen
	
	ldd 	last_rock_ptr
	cmpd 	flashing_mushroom_location
	BNE 	keke
	
	ldd		sprite_screen_ptr
	std		last_rock_ptr
	LDA 	flash_duration
	STA 	flash_counter
keke:
	LDA 	collision_object
	cmpa	[rock_image_ptr] 	; don't get points for eating a flashing mushroom
	BEQ 	ok3
	lbsr	increment_score
	INC 	snake_segment_count

ok3:
	; play the next note in the tune if ready
;	LDA sound_flag
;	BNE ok2
;	LDA #osb_buffer_status
;	LDX #249	; sound buffer 2
;	LDY #0
;	lbsr osbyte
;	CPX #12
;	BMI ok2
;	LDY #0
;	LDA (tune_ptr),Y
;	STA tune_sound
;	INY
;	LDA (tune_ptr),Y
;	STA tune_sound+1
;	LDA #10
;	LDX tune_sound
;	LDY tune_sound+1
;	lbsr sound
;	; add a short pause (electron only)
;	LDA #osw_play_sound
;	LDX #LO(pause_sound)
;	LDY #HI(pause_sound)
;.pause
;	lbsr osword
;	CLC
;	LDA tune_ptr
;	ADC #2
;	STA tune_ptr
;	LDA tune_ptr+1
;	ADC #0
;	STA tune_ptr+1
;	LDY #0
;	LDA (tune_ptr),Y
;	BNE ok2
;	LDA #<tune_data
;	STA tune_ptr
;	LDA #>tune_data
;	STA tune_ptr+1
ok2:
	; level ends after eating all the mushrooms
	LDA 	snake_segment_count
	cmpa	#mushroom_count+initial_snake_segment_count
	BNE 	gone
	lbra	newlev
	
gone:
	LDA 	#game_speed
	BEQ 	fl
	STA 	>counter
gloop:
	lbsr	oswaitsync
	DEC 	>counter
	BNE 	gloop
	; check if snake has just eaten a flashing mushroom

fl:
	ldd 	last_rock_ptr
	cmpd 	snake_head_ptr
	BNE 	nofl
	
	ldd		flashing_mushroom_location
	std 	last_rock_ptr
	LDA 	flash_duration
	STA 	flash_counter

	lbsr	eat_flash_sound1
	lbsr	eat_flash_sound2

nofl:
pau:
;	lda $b001
;	and #$40
;	BNE nopau
	lda		last_key
	cmpa	#key_pause
	bne		nopau
	
xxx:
	tst 	pause_flag
	BNE 	nopau
	inc 	pause_flag
	lbsr 	display_pause_on_screen

; wait for pause key to be released !
pkeyup:
	lbsr 	osrdch
	cmpa	#key_pause
	beq		pkeyup

pauwa:
	lbsr 	osrdch
	cmpa	#key_pause
	BNE 	pauwa
	lbsr	display_pause_off_screen
	clr		pause_flag
	
; check to see if flashing mushroom timer has expired
nopau:	
	DEC 	flash_counter
	LDA 	flash_counter
	BNE 	nopau2
	
	LDA 	flash_duration
	STA 	flash_counter
	ldd 	last_rock_ptr
	std 	sprite_screen_ptr
	
	cmpd	flashing_mushroom_location
	BEQ 	nprock
	lbsr	display_rock ; turn flasher back into a rock

nprock:
	lbsr	place_rock ; make a new flasher at a random position
	ldd 	sprite_screen_ptr
	std		last_rock_ptr
nopau2:
	lbra	eachgo

newlev:
	; 1 sec delay
	LDA 	#wait_time
	STA 	wait_counter
nl2:
	lbsr 	oswaitsync
	DEC 	wait_counter
	BNE 	nl2
	
	; add 200 bonus points to score
	LDA 	#20
	STA 	temp_b
nl3:
	LDA 	#10
	STA 	wait_counter
nl4:
	lbsr 	oswaitsync
	DEC 	wait_counter
	BNE 	nl4

	lbsr	end_level_sound

	lbsr	increment_score
	DEC 	temp_b
	BNE 	nl3
	lbsr	clear
	
	; move to next level + get an extra life
	INC 	current_screen_number
	LDA 	lives_counter
	cmpa	#max_lives_count
	BEQ 	gg
	INC 	lives_counter

gg:
	LDA 	current_screen_number
	cmpa	#max_screen_number+1
	BMI 	ggg
	LDA 	#max_screen_number
	STA 	current_screen_number

ggg:
	lbra	go
	
; clear a life marker from the status panel
dead:	
	LDA 	lives_counter
	ldx		lives_screen_location
	coma	
	leax	a,x
	stx		sprite_screen_ptr
	lbsr	delete_sprite
	; play death sound effect

	lbsr	die_sound

	; 1 sec delay
	LDA 	#wait_time
	STA 	wait_counter
wawa:
	lbsr	oswaitsync
	DEC 	wait_counter
	tst		wait_counter
	BNE 	wawa
	
	; decrement lives, check if no lives left
	DEC 	lives_counter
	LDA 	lives_counter
	BEQ 	dd2
	
	lbsr	clear
	lbra	go
dd2:
	; print Game Over message
	lbsr	display_game_over_screen

	LDA 	#wait_time*2
	STA 	wait_counter
	
	; update highest screen number reached
	LDA 	current_screen_number
	cmpa	last_screen_number
	BMI 	wawa3
	STA 	last_screen_number

	; 1 sec delay
wawa3:
	lbsr	oswaitsync
	DEC 	wait_counter
	tst 	wait_counter
	BNE 	wawa3
	lbsr	clear
	
	; see if score is in high score table
	lbsr	check_high_score
	BCC 	restart 		; if not, just show title screen
	lbsr	edit_high_score ; else enter name
	LDA 	#1 				; then show the high score screen first
	lbra 	show_screen
restart:
	lbra	title_screen

;-------------------------------------------------------------
; find the safe zone around the snake's head
find_safe_zone:	
	; find the left + right limits of the safe zone
	ldx 	snake_head_ptr
	stx 	safe_zone_left_ptr
	LDA 	snake_direction
	cmpa	#dir_right			; if moving right, ignore columns to left of head
	BEQ 	find_right_zone
	ldb 	#safe_zone_size
	
left_loop:
	leax	-1,x
;	stx		safe_zone_left_ptr
	LDA 	$40,x
	cmpa	[wall_image_ptr]
	BNE 	next_left
	
	leax	1,x					; hit the wall
;	stx		safe_zone_left_ptr	
	BNE 	find_right_zone

next_left:
	decb
	BNE 	left_loop
	
find_right_zone:
	stx		safe_zone_left_ptr	
	ldx		snake_head_ptr
	stx		safe_zone_right_ptr
	LDA 	snake_direction
	cmpa	#dir_left			; if moving left, ignore columns to right of head
	BNE 	right1
	ldb 	#1
	BNE 	right_loop
right1:
	LDX 	#safe_zone_size+1	; +1 because the snake head occupies the first slot
right_loop
	leax	1,x
	stx		safe_zone_right_ptr	
	LDA 	$40,x
	cmpa	[wall_image_ptr]
	BEQ 	find_top_zone
	decb
	BNE 	right_loop
	
find_top_zone:
	; move the pointers to the top of the safe zone
	LDA 	snake_direction
	cmpa	#dir_down			; if moving down, ignore rows above head
	BEQ 	done
	
	ldd		safe_zone_left_ptr
	subd	#safe_zone_size
	std		safe_zone_left_ptr
	ldd		safe_zone_right_ptr
	subd 	#safe_zone_size
	std		safe_zone_right_ptr
done:
	RTS

;-------------------------------------------------------------
; place a rock at a random screen location
place_rock:	
	pshs	d,x,y
	sta 	>counter
	bsr		find_safe_zone
	
; make a pointer to a random position on the screen
again:		
	lbsr	get_random_position
	LDY 	sprite_screen_ptr		; don't place the rock if something is already there
	LDA 	$40,Y
	BNE 	again
	
	; don't place the rock in the safe zone around the snake's head
	ldd		safe_zone_left_ptr
	std		>w_temp_store0
	
	ldd		safe_zone_right_ptr
	std		>w_temp_store2
	
	LDA 	snake_direction
	cmpa	#dir_up			; if moving up/down, can ignore the other half of the rows
	BCC 	start
	ldb 	#safe_zone_size+1
	BNE 	loop5
	
start:
	ldb		#safe_zone_size*2+1

; if rock before left of zone, that's ok
loop5:	
	ldd 	sprite_screen_ptr
	cmpd 	>w_temp_store0
	BCC 	ok
	
	; if rock inside safe zone, try again
	ldd 	sprite_screen_ptr
	cmpd	>w_temp_store2
	BCC 	again
	
	; otherwise, try the next line in the safe zone
	inc 	>w_temp_store0
	inc 	>w_temp_store2
	decb
	BNE 	loop5
	; if rock not in safe zone, it's ok to place it
ok:
	lbsr	display_rock
	puls    d,x,y,pc
		
;-------------------------------------------------------------
move_snake_forwards:
	pshs	d,x,u
	ldb 	snake_segment_count
	leau	snake_segment_pointers,pcr
	leau	b,u
	leau	b,u
loop8:
	ldx 	-2,u
	stx 	,u
	leau	-2,u
	decb
	BNE 	loop8
	puls	d,x,u,pc

;-------------------------------------------------------------
get_random_position:
	pshs	d,x
	lbsr	osrnd
	anda	#$1F
	pshs	a
	
rndloop:
    lbsr	osrnd
	lda		#19					; max line no
	bsr		maxrand_a			; get random number
    
	adda	#play_area_top_line
	clrb
	ldx		GrDisplayStartAddr			; base of graphics screen
	leax	d,x					; convert to pointer
	puls	a					; restore x co-ordinate
	leax	a,x

	stx		sprite_screen_ptr
	
	puls	d,x,pc

; get a random number.
; entry : a=max number 
; exit : a=random value
maxrand_a:
	pshs	a
	lbsr	osrnd				; get value 0..255

maxrand_loop:	
	cmpa	,s					; compare to max number
	blo		maxrand_exit		; lower, got it.
	suba	,s					; subtract maximum and try again
	bra		maxrand_loop
	
maxrand_exit:
	leas	1,s					; drop max value from stack
	rts

;-------------------------------------------------------------
seed_random:        
;	lbsr 	osrnd
;	LDA 	$8
;	STA 	random_bits
;	LDA 	$9
;	STA 	random_bits+1
;	LDA 	$a
;	STA 	random_bits+2
;	LDA 	$b
;	STA 	random_bits+3
	RTS

;scr_ptr  .db 0,$20,$40,$60,$80,$a0,$c0,$e0

;-------------------------------------------------------------
;SUBROUTINES
;-------------------------------------------------------------

draw_snake_head:
	pshs	d,x,u
	ldx		sprite_base_ptr
	leax	snake_head_sprite_data,x
	ldu		snake_head_ptr
	bra		do_draw_sprite

draw_snake_head_dir:
	pshs	d,x,u
	ldu		snake_head_ptr
	bra		do_draw_sprite

;---------------------------------------------------------------
draw_snake_segment:
	pshs	d,x,u
	ldx		sprite_base_ptr
	leax	snake_body_sprite_data,x
	bra		do_draw_sprite_ptr

;-------------------------------------------------------------
display_mushroom:
	pshs	d,x,u
	ldx		sprite_base_ptr
	leax	mushroom_sprite_data,x
	bra		do_draw_sprite_ptr

;-------------------------------------------------------------
display_rock:
	pshs	d,x,u
	ldx		sprite_base_ptr
	leax	rock_sprite_data,x
	bra		do_draw_sprite_ptr

;-------------------------------------------------------------
draw_sprite:
	pshs	d,x,u
	ldx		sprite_data_ptr			; get data ptr
do_draw_sprite_ptr:
	ldu		sprite_screen_ptr		; get screen pointer
do_draw_sprite:
	clrb
draw_loop:
	lda		b,x
	STA 	,u
	leau	characters_per_line,u
	incb
	cmpb	#bytes_per_character
	BNE 	draw_loop
	puls	d,x,u,pc


;---------------------------------------------------------------
delete_sprite:
	pshs	d,x,u
	clrb
	ldu		sprite_screen_ptr
delete_snake_loop:
	clr 	,u
	leau	characters_per_line,u
	incb
	cmpb	#bytes_per_character
	BNE 	delete_snake_loop

	puls	d,x,u,pc
	
;---------------------------------------------------------------
; X=Offset from start of screen memory
; A=number of bricks

draw_horizontal_wall:
    pshs	d,x,u
	bsr		offset_to_abs
	stx		sprite_screen_ptr
	ldx		sprite_base_ptr
	leax	brick_wall_sprite_data,x	
	stx		sprite_data_ptr
	
	pshs	a
	sta		>counter
	
draw_horiz_loop:
	lbsr	draw_sprite
	
	ldu		sprite_screen_ptr
	leau	1,u
	stu		sprite_screen_ptr
	
	DEC 	>counter
	BNE 	draw_horiz_loop
	
    puls	a
    sta 	>counter
	puls	d,x,u,pc

;---------------------------------------------------------------
; X=Offset from start of screen memory
; A=number of bricks

draw_vertical_wall:
    pshs	d,x,u
	bsr		offset_to_abs
	stx		sprite_screen_ptr
	pshs	a
    sta 	>counter
	ldx		sprite_base_ptr
	leax	brick_wall_sprite_data,x	
	stx		sprite_data_ptr

draw_vert_loop:
	lbsr	draw_sprite
	
	ldu		sprite_screen_ptr
	leau	bytes_per_line,u
	stu		sprite_screen_ptr
	
	DEC 	>counter
	BNE 	draw_vert_loop
    
	puls	a
    sta 	>counter
	puls	d,x,u,pc

; Convert offset pointer in x to absolute
offset_to_abs:
	pshs	d
	tfr		x,d
	ldx		GrDisplayStartAddr			; Pointer to graphics screen
	leax	d,x					; add offset
	puls	d,pc

;---------------------------------------------------------------
increment_score:
	inc 	score+3,pcr
	lda 	score+3,pcr
	
	cmpa	#10
	BMI 	display_score
	clr		score+3,pcr
	
	inc 	score+2,pcr
	lda 	score+2,pcr
	cmpa	#10
	BMI 	display_score
	clr		score+2,pcr
		
	inc 	score+1,pcr
	lda 	score+1,pcr
	cmpa	#10
	BMI 	display_score
	
	clr		score+1,pcr
	inc 	score,pcr
	
display_score:
	lda 	score,pcr
	adda	#$30
	sta 	score_set+2,pcr
    
	lda 	score+1,pcr
	adda	#$30
	sta		score_set+3,pcr
	
	lda		score+2,pcr
	adda	#$30
	sta 	score_set+4,pcr
	
    lda 	score+3,pcr
	adda	#$30
	sta 	score_set+5,pcr
	
	lbsr	display_score_screen
	RTS

;---------------------------------------------------------------
hs_test:
	clra
	clrb
	std		score
	ldd		#$0202
	std		score+2,pcr
	
	clr		lives_counter
	lbra	dd2

;---------------------------------------------------------------
check_high_score:
; on exit: carry set if new high score, counter = entry position (0-9),  w_temp_store0 = ptr to entry
	leax	>hs1a,pcr			; point at score table, past co-ordinates

    ; Check score with scoretable entries
	ldb		#9					; check this many scores
	stb		>counter
loop38:
	leau	score,pcr			; point at our score
	clrb
loop39:
	LDA 	b,u					; fetch a byte from our score
	ORA 	#'0					; convert to ASCII
	cmpa 	b,x					; does it match ?
	blo		next				; lower , skip to next score
	bhi 	found				; higher, our score is higher, insert here
	incb						; same check next char
	cmpb	#4					; done all chars ?
	bne		loop39				; nope keep going

next:
	leax	<hssize,x			; move to next score
	dec		>counter			; decrement entry count	
	BNE 	loop38				; more highscores to check, keep going
	bra 	donecc				; done all : exit
	
found:
    ; Move remaining entries down
	inc		>counter
	stx		>high_score_entry_ptr	; save score entry pos
	tst		>counter			; Last entry ?
	BEQ 	clr					; yep, no need to move
	
	leax	hs10a,pcr			; point at beginning of last score

hs_copy_loop1:
	leau	<-hssize,x			; point u at previous score
	clrb
	
hs_copy_loop0:
	lda		b,u					; get byte
	sta		b,x					; put in entry below
	incb						; increment byte count
	cmpb	#hscopylen			; done all ?
	bne		hs_copy_loop0		; nope keep going

	cmpu	high_score_entry_ptr	; got back to our entry yet ?
	beq		clr					; yep skip to clear
	leau	<-hssize,u			; move to previous entry
	leax	<-hssize,x			; move to previous entry
	bra		hs_copy_loop1		; do next keep going
	
clr:
    ; Make a new entry
	ldx		>high_score_entry_ptr	; retrieve score entry pos
	leax	hsnameofs,x				; skip to area to clear
	lda		#$20
	clrb							; clear entry
hs_cloop1:
	sta		b,x
	incb
	cmpb	#hsnamelen				; done all ?
	bne		hs_cloop1

	clrb						; offset
	ldx		>high_score_entry_ptr	; retrieve score entry pos
cloop:
	leau	>score,pcr			; point at score
	lda		b,u					; get a character
	ORA 	#'0					; convert to ASCII
	STA 	b,x					; save in table
	incb
	cmpb	#4					; done 5 digits ?
	bne 	cloop
done1:
	orcc	#FlagCarry
	rts
	
donecc:
	andcc	#~FlagCarry
	RTS

;---------------------------------------------------------------
edit_high_score:
	lbsr	display_high_scores
	lbsr	display_congratulations

	LDA 	#0
	STA 	>high_score_crsr_pos
	
	ldx		>high_score_entry_ptr
	leax	hsnameofs,x

key:
	lbsr	osrdch
	cmpa 	#13  ; return key
	BEQ 	edit_done
	
	cmpa	#$7f  ; delete key
	BNE 	notback
	; delete last character
	
	ldb		>high_score_crsr_pos
	BEQ 	key
	
	decb 	
	stb 	>high_score_crsr_pos
	addb	#12
	stb		>XPOS
	lda		#21
	sta		>YPOS
	lda 	#$20		; space
	lbsr 	PUTCHAR
	
	lda 	#32
	ldb		>high_score_crsr_pos
	STA 	b,x
	bra 	key
	
notback:
    ldb		>high_score_crsr_pos		; no more than 8 characters
    cmpb	#8
    beq 	key
	
	cmpa 	#$41   ;A
	blo		key
	cmpa 	#$5A   ;Z
	bhi 	key
	
	STA 	b,x
	addb	#12
	stb		>XPOS
	ldb		#21
	stb		>YPOS
	lbsr 	PUTCHAR
	
;	lda 	>high_score_crsr_pos
;	cmpa 	#7
;	beq 	key
	
	INC 	>high_score_crsr_pos
	bra 	key
edit_done:
	lbsr	display_high_scores
	RTS

;---------------------------------------------------------------
; Display title screen
;---------------------------------------------------------------

display_title_screen:
        lbsr 	clear4
		leax 	>title_screen_text,pcr
        lbra 	PUTSTRING
;---------------------------------------------------------------
; Display score
;---------------------------------------------------------------

display_score_screen:
		leax	>score_set,pcr
        lbra 	PUTSTRING

;---------------------------------------------------------------
; Display Pause on screen
;---------------------------------------------------------------

display_pause_on_screen:
		leax	>pause_on_text,pcr
        lbra 	PUTSTRING

;---------------------------------------------------------------
; Display Pause off screen
;---------------------------------------------------------------

display_pause_off_screen:
		leax 	>pause_off_text,pcr
        lbra 	PUTSTRING

;---------------------------------------------------------------
; Display Game over
;---------------------------------------------------------------

display_game_over_screen:
		leax	>game_over_text,pcr
        lbra	PUTSTRING

;---------------------------------------------------------------
; Display high scores
;---------------------------------------------------------------

display_high_scores:
        lbsr 	clear4
		leax 	>high_score_text,pcr
		lbra 	PUTSTRING

;---------------------------------------------------------------
; Display Press space
;---------------------------------------------------------------

display_press_space:
		leax	>press_space_text,pcr
        lbra	PUTSTRING

;---------------------------------------------------------------
; Display Which screen
;---------------------------------------------------------------

display_which_screen:
		leax	>which_screen_text,pcr
        lbra	PUTSTRING

;---------------------------------------------------------------
; Display status panel
;---------------------------------------------------------------

display_status_panel:
		lbsr 	clear4
		leax	>status_panel_text,pcr
        lbra	PUTSTRING

;---------------------------------------------------------------
; Display Congratulations text
;---------------------------------------------------------------

display_congratulations:
		leax	>congratulations_text,pcr
        lbra	PUTSTRING


;---------------------------------------------------------------
; Clear screen 4
;---------------------------------------------------------------

clear4: 
		pshs	d,x,u
;        lda #$D0        ; Clear4 color
;        sta $b000

		ldx		GrDisplayStartAddr			; Pointer to graphics screen
		ldu		#6*1024				; no of bytes to do
clear4_1:
		clra	
clrloop:
        sta 	,x					; Clear byte
        leax	1,x
		leau	-1,u
		cmpu	#0
		bne		clrloop
        puls	d,x,u,pc

;---------------------------------------------------------------
; Clear playfield
;---------------------------------------------------------------
clear:
		pshs	d,x,u
		ldx		GrDisplayStartAddr			; Pointer to graphics screen
		leax	(4*bytes_per_line),x
		ldu		#(6*1024)-(4*bytes_per_line)	; no of bytes to do
		lbra 	clear4_1

;---------------------------------------------------------------
; Variables
;---------------------------------------------------------------

screen_duration 	fcb		20   ; 10 sec
random_bits     	fdb		$ffff,$ffff

;---------------------------------------------------------------
; Data
;---------------------------------------------------------------
;				 01234567890123456789012345678901
title_screen_text:
		fcb 	red+6,1
		fcc		"*** HYPER VIPER ***"
		fcb		0
        fcb 	blue+2,3
		fcc		"BBC VERSION BY KENTON PRICE"
		fcb		0
        fcb 	yellow+2,4
		fcc		"ATOM VERSION BY KEES VAN OSS"
		fcb		0
		
		fcb		red+6,5
		fcc		"DRAGON / COCO VERSION"
		fcb		0
		fcb		red+7,6
		fcc		"PHILL HARVEY-SMITH"
		fcb		0
		
		fcb 	red+14,8
		fcc		"----"
		fcb		0
        fcb 	red+14,9
		fcc		"KEYS"
		fcb		0
        fcb 	red+14,10
		fcc		"----"
		fcb		0
        fcb 	red+11,11
		fcc		"O ... LEFT"
		fcb		0
        fcb 	red+11,12
		fcc		"P ... RIGHT"
		fcb		0
        fcb 	red+11,13
		fcc		"Q ... UP"
		fcb		0
        fcb 	red+11,14
		fcc		"A ... DOWN"
		fcb		0
        fcb 	red+5,16
		fcc		"CLEAR ... GAME PAUSE"
		fcb		0
        fcb 	$ff

score_set:
		fcb 	yellow+6,0
		fcc		"00000"
		fcb		0
        fcb 	$ff

press_space_text:
        fcb 	blue+6,22
		fcc		"PRESS SPACE TO PLAY"
		fcb		0
        fcb 	$ff

high_score_text:
		fcb 	blue+6,1
		fcc		"*** HYPER VIPERS ***"
		fcb		0
hs1		fcb 	red+8,5
hs1a	fcc		"00100"
hs1b	fcc		"   "
hs1c	fcc		"RETRO..."
hs1d	fcb		0
hs2		fcb 	yellow+8,6
		fcc		"00100   SOFTWARE"
		fcb		0
		fcb 	blue+8,7
		fcc		"00100   RETRO..."
		fcb		0
		fcb 	red+8,8
		fcc		"00100   SOFTWARE"
		fcb		0
		fcb 	yellow+8,9
		fcc		"00100   RETRO..."
		fcb		0
		fcb 	blue+8,10
		fcc		"00100   SOFTWARE"
		fcb		0
		fcb 	red+8,11
		fcc		"00100   RETRO..."
		fcb		0
		fcb 	yellow+8,12
		fcc		"00100   SOFTWARE"
		fcb		0
hs9		fcb 	blue+8,13
		fcc		"00100   RETRO..."
		fcb		0
hs10	fcb 	red+8,14
hs10a	fcc		"00100   SOFTWARE"
		fcb		0
hs11    fcb 	$ff

hsskip		equ		hs1a-hs1
hssize		equ		(hs2-hs1)
hsnamelen	equ		hs1d-hs1c
hsnameofs	equ		hs1c-hs1a
hscopylen	equ		hs1d-hs1a

congratulations_text:
		fcb 	red+8,16
		fcc		"CONGRATULATIONS!"
		fcb		0
new_high_score_text:
		fcb 	yellow+9,17
		fcc		"NEW HIGH SCORE"
		fcb		0
enter_name_text:
        fcb 	blue+5,19
		fcc		"PLEASE ENTER YOUR NAME:"
		fcb		0
        fcb 	$ff

status_panel_text:
		fcb 	red+0,0
		fcc		"SCORE"
		fcb		0
        fcb 	yellow+6,0
		fcc		"00000"
		fcb		0
		fcb 	blue+10,2
		fcc		"HYPER VIPER"
		fcb		0
        fcb 	blue+27,0
		fcc		"LIVES"
		fcb		0
        fcb 	$ff

which_screen_text:
		fcb 	4,17
		fcc		"                       "
		fcb		0
		fcb 	5,18
		fcc		"WHICH SCREEN (1 TO  )"
		fcb		0
		fcb 	4,19
		fcc		"                       "
		fcb		0
		fcb 	$ff

pause_on_text
		fcb 	yellow+2,1
		fcc		"GAME PAUSE, CLEAR CONTINUES"
		fcb		0
		fcb 	$ff

pause_off_text
		fcb 	2,1
		fcc		"                            "
		fcb		0
		fcb 	$ff

game_over_text:
		fcb 	green+10,11
		fcc		"           "
		fcb		0
		fcb 	blue+10,12
		fcc		" GAME OVER "
		fcb		0
		fcb 	green+10,13
		fcc		"           "
		fcb		0
		fcb 	$ff

